#############################################################################
#
# LOGREP - Logfile Extraction And Reporting System                          
# Copyright (C) 2002 Tevfik Karaglle                                   
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
#
#############################################################################

#########
#
# LOGREP - Log reporting system common library
# v1.5 - - Verbose option is revisited, dec 02, TK
#		 - Optimization in GenerateTables, tripled performance
#		 - Optimization in graph generation
#		 - Customizable number of top graph elements
#		 - Filling blanks in graphs with elements less than max.count are not used anymore
#		 - Trim graphtexts to make a better use of graphing space and avoid erroneous input to
#		 graphing module
#		 - Introducing -s (subset parameter list) option
# v1.4 - Verbose option / PrintFeedback function is added (-v), oct 02, TK
# v1.3 - Generate 2D links even if there is only one element, sep 02, TK
# v1.2 - Make sure that graph elements fewer than 10 are treated properly, sep 02, TK
# v1.1 - Use only basename for html links, jun 02, TK
# v1.0 - Initial version, jun 02, Tevfik K. (www.itefix.no)

$FEEDBACKSEC = 3;	# Number of seconds between progress messages if verbose option is ON
$MAXTOPCOUNT = 15;	# maximum number of elements in top graph
$MAXGRAPHSTRLEN = 30; # To make sure that long strings does not occupy available graph space

use CGI qw/*table/;
use GD::Graph::bars;
use Getopt::Std;

%monthnr = (
	"Jan" => "01",
	"Feb" => "02",
	"Mar" => "03",
	"Apr" => "04",
	"May" => "05",
	"Jun" => "06",
	"Jul" => "07",
	"Aug" => "08",
	"Sep" => "09",
	"Oct" => "10",
	"Nov" => "11",
	"Dec" => "12"
);

$dieopenmsg = "Can't open file $!";
$dieclosemsg = "Can't close file $!";

# Sets common parameters for reporting modules
#
sub SetCommonParameters {
	
	$q = new CGI;
	$indexpage = "index.html";
	$cssfile = "logrep.css";

	# Get command line parameters
	getopts("l:w:s:v") or die $!;
	$webdir = $opt_w;
	$logfilename = $opt_l;
	$verbose = $opt_v;

	CheckSubsetList ();			
	
	# Generate table names
	foreach my $x (@dim) {
		foreach my $y (@dim) {	
			next if ($x eq $y);
			$tblname {"$x$y"} = "tbl$x$y";
			$resname {"$x$y"} = ucfirst($x) . "-" . ucfirst($y);	
		}
	}
	
	$startdate = $enddate = "";
		
	# Generate style sheets
	GenerateCSSFile ();
	
	# counter for verbose option	
	$starttime = time;
	$previnterval = 0;
}

# Generates stylesheet for web pages
#
sub GenerateCSSFile {

	open HTMLPAGE, "> $webdir/$cssfile" or die $dieopenmsg;
	
	print HTMLPAGE "A:hover {color : Green;}\n";
	
	print HTMLPAGE "H1 {font-family : Verdana, Geneva, Arial, Helvetica, sans-serif; font-size : x-large;}\n";
	print HTMLPAGE "H2 {font-family : Verdana, Geneva, Arial, Helvetica, sans-serif; font-size : large;}\n";
	print HTMLPAGE "H3 {font-family : Verdana, Geneva, Arial, Helvetica, sans-serif; font-size : medium;}\n";
	print HTMLPAGE "BODY {font-family : Verdana, Geneva, Arial, Helvetica, sans-serif;	font-size : 12; color : #000080; background-color : #FFA07A;}\n";
	print HTMLPAGE "TABLE {font-family : Verdana, Geneva, Arial, Helvetica, sans-serif; font-size : 12; color : #000080;}\n";

	close HTMLPAGE or die $dieclosemsg;
}

# Generates counter tables
#
sub GenerateTables {

	my $tx = my $ty = my $table = 0;
	my $x = my $y = 0;
	
	# 2-D data collection
	foreach $x (@dim) {
	
		$tx = $$x;
		
		foreach $y (@dim) {
					
			next if ($x eq $y);
			
			$ty = $$y;
			
			$table = \%{${$tblname {"$x$y"}}{$tx}};

			($table->{$ty} ) ++;
			($table->{"sum"}) ++;
		}
	}

	# Print progress info if verbose option is on
	if ($verbose) {
	
		$elapsed = time - $starttime; # elapsed time in seconds
		
		# Print only once for each verbose interval
		if (($elapsed != $previnterval) && (($elapsed % $FEEDBACKSEC) == 0)) {
		
			printf "%d (%d lines per sec)\r", $., $. / $elapsed;
			$previnterval = $elapsed;

		}
	}	
}

# WriteIndexPage : 
#	Generates index html page for analysis results
#	Parameter 1 : Main title
#	Parameter 2 : Title for parameters table
#	parameter pair (can be more than one): summary title, summary value
#
sub WriteIndexPage {

	my $summarytitle = shift;
	my $tabletitle = shift;
	
	open HTMLPAGE, "> $webdir/$indexpage" or die $dieopenmsg;
	
	PrintHtmlHeader (HTMLPAGE);	

	print HTMLPAGE
		$q->h1 ($summarytitle),
		$q->p (	
			$q->b ("Filename: "),
			"$logfilename ($n lines, from $startdate to $enddate)"
		);
		
	foreach $summary (@_) {

		my $summarytitle = shift;
		my $summaryvalue = shift;
		
		print HTMLPAGE
			$q->p (
				$q->b ($summarytitle),
				$summaryvalue
			);
	}

	# Print html code for parameter table	
	print HTMLPAGE 
		$q->hr, 
		$q->h2 ($tabletitle);

	print HTMLPAGE '<table border="1">';

	foreach my $x (@dim) {

		print HTMLPAGE "<TR>";
	
		foreach my $y (@dim) {
		
			next if ($x eq $y);
		
			my $htmlpagex = $resname{"$x$y"} . "-x.html";
		
			# Don't generate link if there is only one element
			my $table = $tblname {"$x$y"};
			my $nkeys = keys %$table;
			my $link = ($nkeys > 1) 
					? $q->a ({href=>"$htmlpagex"}, $resname{"$x$y"})
					: $q->font({color=>"808080"},$resname{"$x$y"});

			print HTMLPAGE 
				$q->td (
					$q->div ({align=>"center"}, 
						$q->h3 ($link)
					)
				);	
		}
		
		print HTMLPAGE "</TR>";
	}

	print HTMLPAGE $q->end_table;
	
	print HTMLPAGE $q->end_html;
	
	close HTMLPAGE or die $dieclosemsg;
}

###############################
# WriteDetailsPages : 
#	Generates html pages for details
#
sub WriteDetailsPages {

	my $graphy = ""; 

	foreach my $x (@dim) {
		
		foreach my $y (@dim) {
		
			# Not interested in same parameters in both dimensions
			next if ($x eq $y);
			
			$graphy = $y; # Keep copy of current y for graph generation in the outer loop
			
			# Don't generate file if there is only one element
			my $table = $tblname {"$x$y"};
			my $nkeys = keys %$table;
			
			next if ($nkeys <= 1);

			# Generate file names for details pages
			my $basenamex = $resname{"$x$y"} . "-x.html";
			my $basenamey = $resname{"$x$y"} . "-y.html";
			my $htmlpagex = "$webdir/$basenamex" ;
			my $htmlpagey = "$webdir/$basenamey";
			
			open HTMLPAGEX, "> $htmlpagex" or die $dieopenmsg;
			open HTMLPAGEY, "> $htmlpagey" or die $dieopenmsg;
			
			PrintHtmlHeader (HTMLPAGEX);
			PrintHtmlHeader (HTMLPAGEY);

			# Generate html output for Top N graph
			print HTMLPAGEX 
				$q->h2 (
					"Top graph for ",
					$q->font ( {color=>"#800000"}, 
						$q->em (ucfirst ($x))
					)
				),
				$q->hr,
				$q->p (
					$q->img ({src=>"$x.png",align=>"CENTER"})
				), 
				$q->hr,
				$q->h2 (
					"Details for ",
					$q->font ( {color=>"#800000"}, 
						$q->em (ucfirst ($x))
					),
					" and Links for ",
					$q->font ( {color=>"#800000"}, 
						$q->em (ucfirst ($y))
					)
				),
				'<table border="1"',
				"<tr>",
					$q->td ($q->b ("$x")),
					$q->td ($q->b ("Count/Link")),
				"</tr>";

			# Generate html output for table entries sorted by sum of occurences
			foreach $$x (sort { $$table{$b}{"sum"} <=> $$table{$a}{"sum"} } keys %$table) 	{		
		
				my $sum = $$table{$$x}{"sum"};
				
				my $link = $q->a ({href=>"$basenamey#$$x"}, $sum);
				
				print HTMLPAGEX 
					"<tr>", 
						$q->td ("$$x"), 
						$q->td ($link),
					"</tr>";

				print HTMLPAGEY 
					$q->h2 (
						$q->a ({name=>"$$x"},
							"Details for ",
							$q->font ( {color=>"#800000"}, 
								$q->em (ucfirst ($x))
							),
							" - $$x"
						),
					),
					'<table border="1"',
					"<tr>",
						$q->td ($q->b ("$y")),
						$q->td ($q->b ("Count")),
					"</tr>";
				
				foreach $$y (sort  { $$table{$$x}{$b} <=> $$table{$$x}{$a} } keys %{$$table{$$x}}) {				 
					
					print HTMLPAGEY 
						"<tr>", 
							$q->td ("$$y"),
							$q->td ("$$table{$$x}{$$y}"),
						"</tr>";
				}
				
				print HTMLPAGEY	
					$q->end_table,
					$q->hr;
			}
			print HTMLPAGEX	$q->end_html;
			
			print HTMLPAGEY $q->end_table, $q->end_html;
		
			close HTMLPAGEX or die $dieclosemsg;
			close HTMLPAGEY or die $dieclosemsg;

		}
		
		# remove graph array
		undef @graphdata;
		my $graphxn = 0;
				
		my $tblx = $tblname {"$x$graphy"}; # generate graph for $x dimension, $y has last value in array

		foreach $$x (sort { $$tblx{$b}{"sum"} <=> $$tblx{$a}{"sum"} } keys %$tblx) 	{		
					
			$graphdata [0][$graphxn] = GraphString ($$x);
			$graphdata [1][$graphxn] = $$tblx{$$x}{"sum"};
			
			$graphxn++;
			
			last if ($graphxn == $MAXTOPCOUNT); # Interested in only TOP N

		}
			
		$graph = GD::Graph::bars->new(400, 400);
		$graph->set( 
			x_label => "$x",
			x_labels_vertical => "1",
			y_label => "Count", 
			title => "$x - Top $graphxn", 
			dclrs => ["dblue"]);

		my $gd = $graph->plot(\@graphdata);
		open(IMG, ">$webdir/$x.png") or die "Problems with creating graph image -$!"; 
		binmode IMG; 
		print IMG $gd->png if (defined $gd);
	}
	
}

# PrintHtmlHeader : 
#	Prints a standard html heeader
#	Parameter : filehandle of web page
#
sub PrintHtmlHeader {

	my $fn = shift;
	print $fn $q->start_html(
		-title=>"Report system for logfiles",
		-style=>{"src" => $cssfile}
	);

}

# GraphString :
#	Formats graph strings longer than maximum allowed length
#	Parameter: String to trim
#	Return : Formatted string
sub GraphString {

	my $string = shift;

	return $string if (length ($string) <= $MAXGRAPHSTRLEN);
	
	$n = ($MAXGRAPHSTRLEN - 4) / 2;
	
	return sprintf '%s[..]%s', substr ($string, 0, $n), substr ($string, -$n);
}

# CheckSubsetList :
#   Check if a subset list is specified and verifies contents of the subset list. 
# If everything is Ok, subset list becomes the new complete list.

sub CheckSubsetList {

	my $subsetres = 0;	# No subset
	
	if ($opt_s) {{	# double brackets for last command!!
	
		$subsetres = 1; #Subset list is specified. Must be verified!!
	
		my $ndim = @dim; # get number of parameters in the original complete list
	
		@arr = split /,/, $opt_s, $ndim;
	
		$narr = @arr;
		last if ($narr == 0); # Don't continue if subset list is empty!
		
		# Check if subset is a really subset of the complete list
		my $nok = 0;
		foreach my $x (@arr) { 
			$nok++ if (grep (/$x/, @dim))
		 };
		last if ($narr != $nok); # Don't continue if subset list couldn't be verified
					
		@dim = @arr; # New array is qualified and replaces the complete list
		
		$subsetres = 0; # subset is a complete list now!
	}}
	
	if ($subsetres) { # Wrong subset!!
	
		my $lststr = "";
		foreach my $x (@dim) {$lststr .= "$x,"}; 
		chop $lststr;
		
		print "Wrong subset parameter list : $opt_s.\nA complete list of\n  $lststr\n is assumed\n";
	}
}


